@@ -20,14 +20,14 @@
 
 #include <config-kconf.h> // CMAKE_INSTALL_PREFIX
 
-#include <QtCore/QDate>
-#include <QtCore/QFile>
-#include <QtCore/QTextStream>
-#include <QtCore/QTextCodec>
+#include <QDate>
+#include <QFile>
+#include <QTextStream>
+#include <QTextCodec>
 #include <QUrl>
 #include <QTemporaryFile>
 #include <QCoreApplication>
-#include <QtCore/QDir>
+#include <QDir>
 #include <QProcess>
 #include <QDebug>
 
@@ -38,17 +38,24 @@
 #include <qcommandlineparser.h>
 #include <qcommandlineoption.h>
 
+#include "kconf_update_debug.h"
 #include "kconfigutils.h"
 
+// Convenience wrapper around qCDebug to prefix the output with metadata of
+// the file.
+#define qCDebugFile(CATEGORY) \
+    qCDebug(CATEGORY) << m_currentFilename << ':' << m_lineCount << ":'" << m_line << "': "
+
 class KonfUpdate
 {
 public:
     KonfUpdate(QCommandLineParser *parser);
     ~KonfUpdate();
-    QStringList findUpdateFiles(bool dirtyOnly);
 
-    QTextStream &log();
-    QTextStream &logFileError();
+    KonfUpdate(const KonfUpdate &) = delete;
+    KonfUpdate& operator=(const KonfUpdate &) = delete;
+
+    QStringList findUpdateFiles(bool dirtyOnly);
 
     bool checkFile(const QString &filename);
     void checkGotFile(const QString &_file, const QString &id);
@@ -105,9 +112,9 @@ protected:
 };
 
 KonfUpdate::KonfUpdate(QCommandLineParser *parser)
-    : m_oldConfig1(Q_NULLPTR), m_oldConfig2(Q_NULLPTR), m_newConfig(Q_NULLPTR),
-      m_bCopy(false), m_bOverwrite(false), m_textStream(Q_NULLPTR),
-      m_file(Q_NULLPTR), m_lineCount(-1)
+    : m_oldConfig1(nullptr), m_oldConfig2(nullptr), m_newConfig(nullptr),
+      m_bCopy(false), m_bOverwrite(false), m_textStream(nullptr),
+      m_file(nullptr), m_lineCount(-1)
 {
     bool updateAll = false;
 
@@ -117,6 +124,19 @@ KonfUpdate::KonfUpdate(QCommandLineParser *parser)
     QStringList updateFiles;
 
     m_debug = parser->isSet(QStringLiteral("debug"));
+    if (m_debug) {
+        // The only way to enable debug reliably is through a filter rule.
+        // The category itself is const, so we can't just go around changing
+        // its mode. This can however be overridden by the environment, so
+        // we'll want to have a fallback warning if debug is not enabled
+        // after setting the filter.
+        QLoggingCategory::setFilterRules(QStringLiteral("%1.debug=true").arg(KCONF_UPDATE_LOG().categoryName()));
+        qDebug() << "Automatically enabled the debug logging category" << KCONF_UPDATE_LOG().categoryName();
+        if (!KCONF_UPDATE_LOG().isDebugEnabled()) {
+            qWarning("The debug logging category %s needs to be enabled manually to get debug output",
+                     KCONF_UPDATE_LOG().categoryName());
+        }
+    }
 
     if (parser->isSet(QStringLiteral("testmode"))) {
         QStandardPaths::setTestModeEnabled(true);
@@ -128,7 +148,7 @@ KonfUpdate::KonfUpdate(QCommandLineParser *parser)
         const QString file = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kconf_update/" + parser->value(QStringLiteral("check")));
         if (file.isEmpty()) {
             qWarning("File '%s' not found.", parser->value(QStringLiteral("check")).toLocal8Bit().data());
-            log() << "File '" << parser->value(QStringLiteral("check")) << "' passed on command line not found" << endl;
+            qCDebug(KCONF_UPDATE_LOG) << "File" << parser->value(QStringLiteral("check")) << "passed on command line not found";
             return;
         }
         updateFiles.append(file);
@@ -142,7 +162,7 @@ KonfUpdate::KonfUpdate(QCommandLineParser *parser)
         updateAll = true;
     }
 
-    foreach (const QString& file, updateFiles) {
+    for (const QString& file : qAsConst(updateFiles)) {
         updateFile(file);
     }
 
@@ -166,60 +186,25 @@ KonfUpdate::~KonfUpdate()
     delete m_textStream;
 }
 
-static QTextStream &operator<<(QTextStream &stream, const QStringList &lst)
-{
-    stream << lst.join(QStringLiteral(", "));
-    return stream;
-}
-
-QTextStream &
-KonfUpdate::log()
-{
-    if (!m_textStream) {
-#if 0
-        QString dir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + "kconf_update/log";
-        QDir().mkpath(dir);
-        QString file = dir + "/update.log";
-        m_file = new QFile(file);
-        if (m_file->open(QIODevice::WriteOnly | QIODevice::Append)) {
-            m_textStream = new QTextStream(m_file);
-        } else {
-            // Error
-            m_textStream = new QTextStream(stderr, QIODevice::WriteOnly);
-        }
-#endif
-        m_textStream = new QTextStream(stderr, QIODevice::WriteOnly);
-    }
-
-    (*m_textStream) << QDateTime::currentDateTime().toString(Qt::ISODate) << " ";
-
-    return *m_textStream;
-}
-
-QTextStream &
-KonfUpdate::logFileError()
-{
-    return log() << m_currentFilename << ':' << m_lineCount << ":'" << m_line << "': ";
-}
-
 QStringList KonfUpdate::findUpdateFiles(bool dirtyOnly)
 {
     QStringList result;
 
     const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kconf_update"), QStandardPaths::LocateDirectory);
-    Q_FOREACH (const QString &d, dirs) {
+    for (const QString &d : dirs) {
         const QDir dir(d);
 
         const QStringList fileNames = dir.entryList(QStringList(QStringLiteral("*.upd")));
-        Q_FOREACH (const QString &fileName, fileNames) {
+        for (const QString &fileName : fileNames) {
             const QString file = dir.filePath(fileName);
             QFileInfo info(file);
 
             KConfigGroup cg(m_config, fileName);
-            const QDateTime ctime = QDateTime::fromTime_t(cg.readEntry("ctime", 0u));
-            const QDateTime mtime = QDateTime::fromTime_t(cg.readEntry("mtime", 0u));
+            const QDateTime ctime = QDateTime::fromSecsSinceEpoch(cg.readEntry("ctime", 0u));
+            const QDateTime mtime = QDateTime::fromSecsSinceEpoch(cg.readEntry("mtime", 0u));
             if (!dirtyOnly ||
-                    (ctime != info.created()) || (mtime != info.lastModified())) {
+                    (ctime.isValid() && ctime != info.birthTime()) ||
+                     mtime != info.lastModified()) {
                 result.append(file);
             }
         }
@@ -257,7 +242,9 @@ bool KonfUpdate::checkFile(const QString &filename)
         }
         if (line.startsWith(QLatin1String("Id="))) {
             if (!foundVersion) {
-                qDebug() << QStringLiteral("Missing \"Version=5\", file \'%1\' will be skipped.").arg(filename);
+                qCDebug(KCONF_UPDATE_LOG,
+                        "Missing 'Version=5', file '%s' will be skipped.",
+                        qUtf8Printable(filename));
                 return true;
             }
             id = m_currentFilename + ':' + line.mid(3);
@@ -323,7 +310,7 @@ bool KonfUpdate::updateFile(const QString &filename)
         return false;
     }
 
-    log() << "Checking update-file '" << filename << "' for new updates" << endl;
+    qCDebug(KCONF_UPDATE_LOG) << "Checking update-file" << filename << "for new updates";
 
     QTextStream ts(&file);
     ts.setCodec(QTextCodec::codecForName("ISO-8859-1"));
@@ -341,7 +328,9 @@ bool KonfUpdate::updateFile(const QString &filename)
         }
         if (m_line.startsWith(QLatin1String("Id="))) {
             if (!foundVersion) {
-                qDebug() << QStringLiteral("Missing \"Version=5\", file \'%1\' will be skipped.").arg(filename);
+                qCDebug(KCONF_UPDATE_LOG,
+                        "Missing 'Version=5', file '%s' will be skipped.",
+                        qUtf8Printable(filename));
                 break;
             }
             gotId(m_line.mid(3));
@@ -376,7 +365,7 @@ bool KonfUpdate::updateFile(const QString &filename)
             gotAllGroups();
             resetOptions();
         } else {
-            logFileError() << "Parse error" << endl;
+            qCDebugFile(KCONF_UPDATE_LOG) << "Parse error";
         }
     }
     // Flush.
@@ -384,8 +373,10 @@ bool KonfUpdate::updateFile(const QString &filename)
 
     QFileInfo info(filename);
     KConfigGroup cg(m_config, m_currentFilename);
-    cg.writeEntry("ctime", info.created().toTime_t());
-    cg.writeEntry("mtime", info.lastModified().toTime_t());
+    if (info.birthTime().isValid()) {
+        cg.writeEntry("ctime", info.birthTime().toSecsSinceEpoch());
+    }
+    cg.writeEntry("mtime", info.lastModified().toSecsSinceEpoch());
     cg.sync();
     return true;
 }
@@ -420,9 +411,9 @@ void KonfUpdate::gotId(const QString &_id)
         m_skipFile = false;
         m_id = _id;
         if (m_bUseConfigInfo) {
-            log() << m_currentFilename << ": Checking update '" << _id << "'" << endl;
+            qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Checking update" << _id;
         } else {
-            log() << m_currentFilename << ": Found new update '" << _id << "'" << endl;
+            qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Found new update" << _id;
         }
     }
 }
@@ -435,7 +426,7 @@ void KonfUpdate::gotFile(const QString &_file)
     if (!m_oldFile.isEmpty()) {
         // Close old file.
         delete m_oldConfig1;
-        m_oldConfig1 = Q_NULLPTR;
+        m_oldConfig1 = nullptr;
 
         KConfigGroup cg(m_oldConfig2, "$Version");
         QStringList ids = cg.readEntry("update_info", QStringList());
@@ -446,7 +437,7 @@ void KonfUpdate::gotFile(const QString &_file)
         }
         cg.sync();
         delete m_oldConfig2;
-        m_oldConfig2 = Q_NULLPTR;
+        m_oldConfig2 = nullptr;
 
         QString file = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1Char('/') + m_oldFile;
         QFileInfo info(file);
@@ -468,11 +459,11 @@ void KonfUpdate::gotFile(const QString &_file)
         }
         m_newConfig->sync();
         delete m_newConfig;
-        m_newConfig = Q_NULLPTR;
+        m_newConfig = nullptr;
 
         m_newFile.clear();
     }
-    m_newConfig = Q_NULLPTR;
+    m_newConfig = nullptr;
 
     int i = _file.indexOf(',');
     if (i == -1) {
@@ -493,7 +484,7 @@ void KonfUpdate::gotFile(const QString &_file)
         if (ids.contains(cfg_id)) {
             m_skip = true;
             m_newFile.clear();
-            log() << m_currentFilename << ": Skipping update '" << m_id << "'" << endl;
+            qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Skipping update" << m_id;
         }
 
         if (!m_newFile.isEmpty()) {
@@ -502,7 +493,7 @@ void KonfUpdate::gotFile(const QString &_file)
             ids = cg.readEntry("update_info", QStringList());
             if (ids.contains(cfg_id)) {
                 m_skip = true;
-                log() << m_currentFilename << ": Skipping update '" << m_id << "'" << endl;
+                qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Skipping update" << m_id;
             }
         } else {
             m_newConfig = m_oldConfig2;
@@ -519,10 +510,10 @@ void KonfUpdate::gotFile(const QString &_file)
 
     m_skipFile = false;
     if (!m_oldFile.isEmpty()) { // if File= is specified, it doesn't exist, is empty or contains only kconf_update's [$Version] group, skip
-        if (m_oldConfig1 != Q_NULLPTR
+        if (m_oldConfig1 != nullptr
                 && (m_oldConfig1->groupList().isEmpty()
                     || (m_oldConfig1->groupList().count() == 1 && m_oldConfig1->groupList().at(0) == QLatin1String("$Version")))) {
-            log() << m_currentFilename << ": File '" << m_oldFile << "' does not exist or empty, skipping" << endl;
+            qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": File" << m_oldFile << "does not exist or empty, skipping";
             m_skipFile = true;
         }
     }
@@ -534,7 +525,7 @@ QStringList KonfUpdate::parseGroupString(const QString &str)
     QString error;
     QStringList lst = KConfigUtils::parseGroupString(str, &ok, &error);
     if (!ok) {
-        logFileError() << error;
+        qCDebugFile(KCONF_UPDATE_LOG) << error;
     }
     return lst;
 }
@@ -561,7 +552,7 @@ void KonfUpdate::gotRemoveGroup(const QString &_group)
     m_oldGroup = parseGroupString(_group);
 
     if (!m_oldConfig1) {
-        logFileError() << "RemoveGroup without previous File specification" << endl;
+        qCDebugFile(KCONF_UPDATE_LOG) << "RemoveGroup without previous File specification";
         return;
     }
 
@@ -571,7 +562,7 @@ void KonfUpdate::gotRemoveGroup(const QString &_group)
     }
     // Delete group.
     cg.deleteGroup();
-    log() << m_currentFilename << ": RemoveGroup removes group " << m_oldFile << ":" << m_oldGroup << endl;
+    qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": RemoveGroup removes group" << m_oldFile << ":" << m_oldGroup;
 }
 
 void KonfUpdate::gotKey(const QString &_key)
@@ -587,11 +578,11 @@ void KonfUpdate::gotKey(const QString &_key)
     }
 
     if (oldKey.isEmpty() || newKey.isEmpty()) {
-        logFileError() << "Key specifies invalid key" << endl;
+        qCDebugFile(KCONF_UPDATE_LOG) << "Key specifies invalid key";
         return;
     }
     if (!m_oldConfig1) {
-        logFileError() << "Key without previous File specification" << endl;
+        qCDebugFile(KCONF_UPDATE_LOG) << "Key without previous File specification";
         return;
     }
     copyOrMoveKey(m_oldGroup, oldKey, m_newGroup, newKey);
@@ -601,7 +592,7 @@ void KonfUpdate::copyOrMoveKey(const QStringList &srcGroupPath, const QString &s
 {
     KConfigGroup dstCg = KConfigUtils::openGroup(m_newConfig, dstGroupPath);
     if (!m_bOverwrite && dstCg.hasKey(dstKey)) {
-        log() << m_currentFilename << ": Skipping " << m_newFileName << ":" << dstCg.name() << ":" << dstKey << ", already exists." << endl;
+        qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Skipping" << m_newFileName << ":" << dstCg.name() << ":" << dstKey << ", already exists.";
         return;
     }
 
@@ -610,7 +601,7 @@ void KonfUpdate::copyOrMoveKey(const QStringList &srcGroupPath, const QString &s
         return;
     }
     QString value = srcCg.readEntry(srcKey, QString());
-    log() << m_currentFilename << ": Updating " << m_newFileName << ":" << dstCg.name() << ":" << dstKey << " to '" << value << "'" << endl;
+    qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Updating" << m_newFileName << ":" << dstCg.name() << ":" << dstKey << "to" << value;
     dstCg.writeEntry(dstKey, value);
 
     if (m_bCopy) {
@@ -625,7 +616,7 @@ void KonfUpdate::copyOrMoveKey(const QStringList &srcGroupPath, const QString &s
     }
     KConfigGroup srcCg2 = KConfigUtils::openGroup(m_oldConfig2, srcGroupPath);
     srcCg2.deleteEntry(srcKey);
-    log() << m_currentFilename << ": Removing " << m_oldFile << ":" << srcCg2.name() << ":" << srcKey << ", moved." << endl;
+    qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Removing" << m_oldFile << ":" << srcCg2.name() << ":" << srcKey << ", moved.";
 }
 
 void KonfUpdate::copyOrMoveGroup(const QStringList &srcGroupPath, const QStringList &dstGroupPath)
@@ -633,12 +624,14 @@ void KonfUpdate::copyOrMoveGroup(const QStringList &srcGroupPath, const QStringL
     KConfigGroup cg = KConfigUtils::openGroup(m_oldConfig1, srcGroupPath);
 
     // Keys
-    Q_FOREACH (const QString &key, cg.keyList()) {
+    const QStringList lstKeys = cg.keyList();
+    for (const QString &key : lstKeys) {
         copyOrMoveKey(srcGroupPath, key, dstGroupPath, key);
     }
 
     // Subgroups
-    Q_FOREACH (const QString &group, cg.groupList()) {
+    const QStringList lstGroup = cg.groupList();
+    for (const QString &group : lstGroup) {
         const QStringList groupPath(group);
         copyOrMoveGroup(srcGroupPath + groupPath, dstGroupPath + groupPath);
     }
@@ -649,12 +642,12 @@ void KonfUpdate::gotRemoveKey(const QString &_key)
     QString key = _key.trimmed();
 
     if (key.isEmpty()) {
-        logFileError() << "RemoveKey specifies invalid key" << endl;
+        qCDebugFile(KCONF_UPDATE_LOG) << "RemoveKey specifies invalid key";
         return;
     }
 
     if (!m_oldConfig1) {
-        logFileError() << "Key without previous File specification" << endl;
+        qCDebugFile(KCONF_UPDATE_LOG) << "Key without previous File specification";
         return;
     }
 
@@ -662,20 +655,20 @@ void KonfUpdate::gotRemoveKey(const QString &_key)
     if (!cg1.hasKey(key)) {
         return;
     }
-    log() << m_currentFilename << ": RemoveKey removes " << m_oldFile << ":" << m_oldGroup << ":" << key << endl;
+    qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": RemoveKey removes" << m_oldFile << ":" << m_oldGroup << ":" << key;
 
     // Delete old entry
     KConfigGroup cg2 = KConfigUtils::openGroup(m_oldConfig2, m_oldGroup);
     cg2.deleteEntry(key);
     /*if (m_oldConfig2->deleteGroup(m_oldGroup, KConfig::Normal)) { // Delete group if empty.
-       log() << m_currentFilename << ": Removing empty group " << m_oldFile << ":" << m_oldGroup << endl;
+       qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Removing empty group " << m_oldFile << ":" << m_oldGroup;
     }   (this should be automatic)*/
 }
 
 void KonfUpdate::gotAllKeys()
 {
     if (!m_oldConfig1) {
-        logFileError() << "AllKeys without previous File specification" << endl;
+        qCDebugFile(KCONF_UPDATE_LOG) << "AllKeys without previous File specification";
         return;
     }
 
@@ -685,7 +678,7 @@ void KonfUpdate::gotAllKeys()
 void KonfUpdate::gotAllGroups()
 {
     if (!m_oldConfig1) {
-        logFileError() << "AllGroups without previous File specification" << endl;
+        qCDebugFile(KCONF_UPDATE_LOG) << "AllGroups without previous File specification";
         return;
     }
 
@@ -733,7 +726,8 @@ void KonfUpdate::copyGroup(const KConfigGroup &cg1, KConfigGroup &cg2)
     }
 
     // Copy subgroups
-    Q_FOREACH (const QString &group, cg1.groupList()) {
+    const QStringList lstGroup = cg1.groupList();
+    for (const QString &group : lstGroup) {
         copyGroup(&cg1, group, &cg2, group);
     }
 }
@@ -755,12 +749,12 @@ void KonfUpdate::gotScript(const QString &_script)
     }
 
     if (script.isEmpty()) {
-        logFileError() << "Script fails to specify filename";
+        qCDebugFile(KCONF_UPDATE_LOG) << "Script fails to specify filename";
         m_skip = true;
         return;
     }
 
-    QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kconf_update/") + script);
+    QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("kconf_update/") + script);
     if (path.isEmpty()) {
         if (interpreter.isEmpty()) {
             path = CMAKE_INSTALL_PREFIX "/" LIB_INSTALL_DIR "/kconf_update_bin/" + script;
@@ -770,16 +764,16 @@ void KonfUpdate::gotScript(const QString &_script)
         }
 
         if (path.isEmpty()) {
-            logFileError() << "Script '" << script << "' not found" << endl;
+            qCDebugFile(KCONF_UPDATE_LOG) << "Script" << script << "not found";
             m_skip = true;
             return;
         }
     }
 
     if (!m_arguments.isNull()) {
-        log() << m_currentFilename << ": Running script '" << script << "' with arguments '" << m_arguments << "'" << endl;
+        qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Running script" << script << "with arguments" << m_arguments;
     } else {
-        log() << m_currentFilename << ": Running script '" << script << "'" << endl;
+        qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Running script" << script;
     }
 
     QStringList args;
@@ -789,7 +783,7 @@ void KonfUpdate::gotScript(const QString &_script)
     } else {
         QString interpreterPath = QStandardPaths::findExecutable(interpreter);
         if (interpreterPath.isEmpty()) {
-            logFileError() << "Cannot find interpreter '" << interpreter << "'." << endl;
+            qCDebugFile(KCONF_UPDATE_LOG) << "Cannot find interpreter" << interpreter;
             m_skip = true;
             return;
         }
@@ -814,7 +808,7 @@ void KonfUpdate::gotScript(const QString &_script)
     if (m_oldConfig1) {
         if (m_debug) {
             scriptIn.setAutoRemove(false);
-            log() << "Script input stored in " << scriptIn.fileName() << endl;
+            qCDebug(KCONF_UPDATE_LOG) << "Script input stored in" << scriptIn.fileName();
         }
         KConfig cfg(scriptIn.fileName(), KConfig::SimpleConfig);
 
@@ -833,16 +827,17 @@ void KonfUpdate::gotScript(const QString &_script)
         }
         cfg.sync();
     }
+
+    qCDebug(KCONF_UPDATE_LOG) << "About to run" << cmd;
     if (m_debug) {
-        log() << "About to run " << cmd << endl;
         QFile scriptFile(path);
         if (scriptFile.open(QIODevice::ReadOnly)) {
-            log() << "Script contents is:" << endl << scriptFile.readAll() << endl;
+            qCDebug(KCONF_UPDATE_LOG) << "Script contents is:" << endl << scriptFile.readAll();
         }
     }
     proc.start(cmd, args);
     if (!proc.waitForFinished(60000)) {
-        logFileError() << "update script did not terminate within 60 seconds: " << cmd << endl;
+        qCDebugFile(KCONF_UPDATE_LOG) << "update script did not terminate within 60 seconds:" << cmd;
         m_skip = true;
         return;
     }
@@ -854,19 +849,17 @@ void KonfUpdate::gotScript(const QString &_script)
         ts.setCodec(QTextCodec::codecForName("UTF-8"));
         while (!ts.atEnd()) {
             QString line = ts.readLine();
-            log() << "[Script] " << line << endl;
+            qCDebug(KCONF_UPDATE_LOG) << "[Script]" << line;
         }
     }
     proc.close();
 
     if (result) {
-        log() << m_currentFilename << ": !! An error occurred while running '" << cmd << "'" << endl;
+        qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": !! An error occurred while running" << cmd;
         return;
     }
 
-    if (m_debug) {
-        log() << "Successfully ran " << cmd << endl;
-    }
+    qCDebug(KCONF_UPDATE_LOG) << "Successfully ran" << cmd;
 
     if (!m_oldConfig1) {
         return; // Nothing to merge
@@ -874,10 +867,10 @@ void KonfUpdate::gotScript(const QString &_script)
 
     if (m_debug) {
         scriptOut.setAutoRemove(false);
-        log() << "Script output stored in " << scriptOut.fileName() << endl;
+        qCDebug(KCONF_UPDATE_LOG) << "Script output stored in" << scriptOut.fileName();
         QFile output(scriptOut.fileName());
         if (output.open(QIODevice::ReadOnly)) {
-            log() << "Script output is:" << endl << output.readAll() << endl;
+            qCDebug(KCONF_UPDATE_LOG) << "Script output is:" << endl << output.readAll();
         }
     }
 
@@ -903,9 +896,9 @@ void KonfUpdate::gotScript(const QString &_script)
                     }
                     KConfigGroup cg = KConfigUtils::openGroup(m_oldConfig2, group);
                     cg.deleteEntry(key);
-                    log() << m_currentFilename << ": Script removes " << m_oldFile << ":" << group << ":" << key << endl;
+                    qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Script removes" << m_oldFile << ":" << group << ":" << key;
                     /*if (m_oldConfig2->deleteGroup(group, KConfig::Normal)) { // Delete group if empty.
-                       log() << m_currentFilename << ": Removing empty group " << m_oldFile << ":" << group << endl;
+                       qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Removing empty group " << m_oldFile << ":" << group;
                     } (this should be automatic)*/
                 } else if (line.startsWith(QLatin1String("# DELETEGROUP"))) {
                     QString str = line.mid(13).trimmed();
@@ -914,7 +907,7 @@ void KonfUpdate::gotScript(const QString &_script)
                     }
                     KConfigGroup cg = KConfigUtils::openGroup(m_oldConfig2, group);
                     cg.deleteGroup();
-                    log() << m_currentFilename << ": Script removes group " << m_oldFile << ":" << group << endl;
+                    qCDebug(KCONF_UPDATE_LOG) << m_currentFilename << ": Script removes group" << m_oldFile << ":" << group;
                 }
             }
         }
@@ -931,7 +924,8 @@ void KonfUpdate::gotScript(const QString &_script)
         KConfigGroup dstCg = KConfigUtils::openGroup(m_newConfig, m_newGroup);
         copyGroup(srcCg, dstCg);
     }
-    Q_FOREACH (const QString &group, scriptOutConfig.groupList()) {
+    const QStringList lstGroup = scriptOutConfig.groupList();
+    for (const QString &group : lstGroup) {
         copyGroup(&scriptOutConfig, group, m_newConfig, group);
     }
 }
